@@ -11,7 +11,9 @@
 #define IWORKTABLE_H_INCLUDED
 
 #include <deque>
+#include <map>
 #include <memory>
+#include <utility>
 
 #include <boost/optional.hpp>
 
@@ -63,6 +65,7 @@ public:
   void setRecorder(const std::shared_ptr<IWORKTableRecorder> &recorder);
   const std::shared_ptr<IWORKTableRecorder> &getRecorder() const;
 
+  void setName(std::string const &name);
   void setSize(unsigned columns, unsigned rows);
   void setHeaders(unsigned headerColumns, unsigned headerRows, unsigned footerRows);
   void setBandedRows(bool banded = true);
@@ -74,6 +77,7 @@ public:
   void setBorders(const IWORKGridLineMap_t &verticalLines, const IWORKGridLineMap_t &horizontalLines);
   void setBorders(const IWORKGridLineMap_t &verticalLeftLines, const IWORKGridLineMap_t &verticalRightLines,
                   const IWORKGridLineMap_t &horizontalTopLines, const IWORKGridLineMap_t &horizontalBottomLines);
+  void setComment(unsigned column, unsigned row, IWORKOutputElements const &text);
   void insertCell(unsigned column, unsigned row,
                   const boost::optional<std::string> &value = boost::none,
                   const std::shared_ptr<IWORKText> &text = std::shared_ptr<IWORKText>(),
@@ -100,12 +104,17 @@ public:
 private:
   IWORKStylePtr_t getDefaultStyle(unsigned column, unsigned row, const IWORKStylePtr_t *group) const;
 
+  boost::optional<std::string> writeFormat(IWORKOutputElements &elements, const IWORKStylePtr_t &style, const IWORKCellType type, boost::optional<std::string> &rvngValueType);
+
 private:
   const IWORKTableNameMapPtr_t m_tableNameMap;
   const IWORKLanguageManager &m_langManager;
+  std::map<librevenge::RVNGString,std::string> m_formatNameMap;
+  std::map<std::pair<unsigned, unsigned>, IWORKOutputElements> m_commentMap;
 
   Table_t m_table;
   IWORKStylePtr_t m_style;
+  boost::optional<std::string> m_name;
   boost::optional<int> m_order;
   IWORKColumnSizes_t m_columnSizes;
   IWORKRowSizes_t m_rowSizes;
