@@ -108,9 +108,14 @@ void IWORKText::fillCharPropList(const IWORKStyleStack &style, const IWORKLangua
       break;
     }
   }
-  else if (style.has<BaselineShift>())
+  else if (style.has<BaselineShift>() && (style.get<BaselineShift>()<0 || style.get<BaselineShift>()>0) &&
+           style.has<FontSize>() && style.get<FontSize>()>0)
   {
-    props.insert("style:text-position", style.get<BaselineShift>(), librevenge::RVNG_PERCENT);
+    // the unit is in point => we need to convert it in a percentage of the font size
+    librevenge::RVNGString str;
+    str.sprintf("%.2f", 100*style.get<BaselineShift>()/style.get<FontSize>());
+    str.append("% 100%");
+    props.insert("style:text-position", str.cstr());
   }
 
   if (style.has<Capitalization>())
@@ -691,7 +696,6 @@ void IWORKText::flushParagraph()
     closeSpan();
   if (m_inPara)
     closePara();
-  m_ignoreEmptyPara = false;
 }
 
 void IWORKText::setSpanStyle(const IWORKStylePtr_t &style)
@@ -778,6 +782,7 @@ void IWORKText::openLink(const std::string &url)
     m_oldSpanStyle = m_spanStyle;
     closeSpan(); // A link is always outside of a span
   }
+  if (m_inLink) closeLink();
 
   // TODO: handle link overflowing to next paragraph
   librevenge::RVNGPropertyList props;
@@ -795,6 +800,11 @@ void IWORKText::closeLink()
     return;
   }
 
+  if (!m_inLink)
+  {
+    ETONYEK_DEBUG_MSG(("IWORKText::closeLink: unexpected called\n"));
+    return;
+  }
   if (m_inSpan)
     closeSpan();
   m_spanStyle = m_oldSpanStyle;
@@ -940,7 +950,7 @@ void IWORKText::handleListLevelChange(const unsigned level)
     fillParaPropList(paraProps, false);
     for (; newLevel > m_inListLevel;)
     {
-      if (m_inListLevel)
+      if (m_inListLevel && !m_inListElement)
         m_elements.addOpenListElement(paraProps);
       ++m_inListLevel;
       RVNGPropertyList listProps;
@@ -949,6 +959,7 @@ void IWORKText::handleListLevelChange(const unsigned level)
         m_elements.addOpenOrderedListLevel(listProps);
       else
         m_elements.addOpenUnorderedListLevel(listProps);
+      m_inListElement=false;
     }
   }
   if (newLevel < m_inListLevel)
@@ -1004,14 +1015,10 @@ void IWORKText::closePara()
 
   if (m_inSpan)
     closeSpan();
+  if (m_inLink)
+    closeLink();
 
-  // TODO: This is a temporary hack. The use of list element vs. paragraph needs rework.
-  if (m_inListLevel > 0)
-  {
-    m_elements.addCloseListElement();
-    m_inListElement=false;
-  }
-  else
+  if (m_inListLevel == 0)
     m_elements.addCloseParagraph();
   m_inPara = false;
 }
