@@ -243,7 +243,8 @@ IWORKGradient::IWORKGradient()
 }
 
 IWORKShadow::IWORKShadow()
-  : m_color()
+  : m_visible(true)
+  , m_color()
   , m_angle(0)
   , m_glossiness(0)
   , m_offset(0)
@@ -281,10 +282,10 @@ IWORKPageMaster::IWORKPageMaster()
 IWORKNumberFormat::IWORKNumberFormat()
   : m_type(IWORK_CELL_NUMBER_TYPE_DOUBLE)
   , m_string()
-  , m_decimalPlaces(0)
+  , m_decimalPlaces(-1)
   , m_currencyCode()
   , m_negativeStyle(0)
-  , m_thousandsSeperator(false)
+  , m_thousandsSeparator(false)
   , m_fractionAccuracy(0)
   , m_accountingStyle(false)
   , m_base(0)
@@ -293,6 +294,25 @@ IWORKNumberFormat::IWORKNumberFormat()
 {
 }
 
+std::string IWORKNumberFormat::getRVNGValueType() const
+{
+  switch (m_type)
+  {
+  case IWORK_CELL_NUMBER_TYPE_CURRENCY:
+    return "currency";
+  case IWORK_CELL_NUMBER_TYPE_DOUBLE:
+    return "double";
+  case IWORK_CELL_NUMBER_TYPE_FRACTION:
+    return "fraction";
+  case IWORK_CELL_NUMBER_TYPE_PERCENTAGE:
+    return "percentage";
+  case IWORK_CELL_NUMBER_TYPE_SCIENTIFIC:
+    return "scientific";
+  default:
+    return "number";
+  }
+}
+
 IWORKDateTimeData::IWORKDateTimeData()
   : m_year(0), m_month(1), m_day(0),
     m_hour(0), m_minute(0), m_second(0)
