@@ -12,12 +12,14 @@
 
 #include "IWORKXMLParserState.h"
 
+#include <IWORKText_fwd.h>
+
 namespace libetonyek
 {
 
 class KEY2Parser;
 class KEYCollector;
-struct KEYDictionary;
+struct KEY2Dictionary;
 
 class KEY2ParserState : public IWORKXMLParserState
 {
@@ -26,14 +28,33 @@ class KEY2ParserState : public IWORKXMLParserState
   KEY2ParserState &operator=(const KEY2ParserState &);
 
 public:
-  KEY2ParserState(KEY2Parser &parser, KEYCollector *collector, KEYDictionary &dict);
+  KEY2ParserState(KEY2Parser &parser, KEYCollector &collector, KEY2Dictionary &dict);
 
-  KEYDictionary &getDictionary();
-  KEYCollector *getCollector();
+  void setVersion(unsigned version);
+  unsigned getVersion() const;
+
+  KEY2Dictionary &getDictionary();
+  KEYCollector &getCollector();
+
+  // Keynote v2: the title and body content is stored in bullets element
+  void openBullets();
+  void openHeadline(int depth);
+  void closeHeadline();
+  void closeBullets();
+
+  IWORKTextPtr_t getBodyText();
+  IWORKTextPtr_t getTitleText();
 
 private:
-  KEYDictionary &m_dict;
-  KEYCollector *const m_collector;
+  unsigned m_version;
+  KEY2Dictionary &m_dict;
+  KEYCollector &m_collector;
+
+  bool m_isHeadlineOpened;
+  bool m_isBulletsOpened;
+
+  IWORKTextPtr_t m_bodyText;
+  IWORKTextPtr_t m_titleText;
 };
 
 }
