@@ -39,6 +39,148 @@ using boost::lexical_cast;
 using boost::optional;
 using std::shared_ptr;
 using std::string;
+namespace
+{
+
+class CellCoordinates: public IWORKXMLEmptyContextBase
+{
+public:
+  CellCoordinates(IWORKXMLParserState &state, boost::optional<std::pair<unsigned,unsigned> > &coordinates)
+    : IWORKXMLEmptyContextBase(state)
+    , m_column()
+    , m_row()
+    , m_coordinates(coordinates)
+  {
+  }
+protected:
+  void attribute(int name, const char *value) final;
+  void endOfElement() final;
+
+  boost::optional<unsigned> m_column, m_row;
+  boost::optional<std::pair<unsigned, unsigned> > &m_coordinates;
+};
+
+void CellCoordinates::attribute(const int name, const char *const value)
+{
+  switch (name)
+  {
+  case IWORKToken::column | IWORKToken::NS_URI_SF :
+    m_column=lexical_cast<unsigned>(value);
+    break;
+  case IWORKToken::row | IWORKToken::NS_URI_SF :
+    m_row=lexical_cast<unsigned>(value);
+    break;
+  default : // none
+    IWORKXMLEmptyContextBase::attribute(name, value);
+  }
+}
+
+void CellCoordinates::endOfElement()
+{
+  if (m_column && m_row)
+    m_coordinates=std::make_pair(*m_column,*m_row);
+  else
+  {
+    ETONYEK_DEBUG_MSG(("CellCoordinates[IWORKTabularModelElement.cpp]::endOfElement: uncomplet data\n"));
+  }
+}
+class CellCommentMappingKey: public IWORKXMLEmptyContextBase
+{
+public:
+  CellCommentMappingKey(IWORKXMLParserState &state, boost::optional<std::pair<unsigned,unsigned> > &coordinates)
+    : IWORKXMLEmptyContextBase(state)
+    , m_coordinates(coordinates)
+  {
+  }
+protected:
+  // attribute: sfa:ID sfa:class
+  IWORKXMLContextPtr_t element(int name) final;
+
+  boost::optional<std::pair<unsigned, unsigned> > &m_coordinates;
+};
+
+IWORKXMLContextPtr_t CellCommentMappingKey::element(int name)
+{
+  switch (name)
+  {
+  case IWORKToken::cell_coordinates | IWORKToken::NS_URI_SF :
+    return std::make_shared<CellCoordinates>(getState(), m_coordinates);
+  default:
+    return IWORKXMLEmptyContextBase::element(name);
+  }
+}
+
+class CellCommentMappingPair: public IWORKXMLEmptyContextBase
+{
+public:
+  CellCommentMappingPair(IWORKXMLParserState &state, std::map<std::pair<unsigned,unsigned>,ID_t> &coordinateCommentRefMap)
+    : IWORKXMLEmptyContextBase(state)
+    , m_coordinates()
+    , m_ref()
+    , m_coordinateCommentRefMap(coordinateCommentRefMap)
+  {
+  }
+protected:
+  // attribute: none
+  IWORKXMLContextPtr_t element(int name) final;
+  void endOfElement() final;
+
+  boost::optional<std::pair<unsigned, unsigned> > m_coordinates;
+  boost::optional<ID_t> m_ref;
+  std::map<std::pair<unsigned,unsigned>,ID_t> &m_coordinateCommentRefMap;
+};
+
+IWORKXMLContextPtr_t CellCommentMappingPair::element(int name)
+{
+  switch (name)
+  {
+  case IWORKToken::value_ref | IWORKToken::NS_URI_SFA : // attributes: sfa:IDREF and sfa:class
+    return std::make_shared<IWORKRefContext>(getState(), m_ref);
+  case IWORKToken::key | IWORKToken::NS_URI_SFA :
+    return std::make_shared<CellCommentMappingKey>(getState(), m_coordinates);
+  default:
+    return IWORKXMLEmptyContextBase::element(name);
+  }
+}
+
+void CellCommentMappingPair::endOfElement()
+{
+  if (m_coordinates && m_ref)
+    m_coordinateCommentRefMap[*m_coordinates]=*m_ref;
+  else
+  {
+    ETONYEK_DEBUG_MSG(("CellCommentMappingPair[IWORKTabularModelElement.cpp]::endOfElement: uncomplete data\n"));
+  }
+}
+
+class CellCommentMapping: public IWORKXMLEmptyContextBase
+{
+public:
+  CellCommentMapping(IWORKXMLParserState &state, std::map<std::pair<unsigned,unsigned>,ID_t> &coordinateCommentRefMap)
+    : IWORKXMLEmptyContextBase(state)
+    , m_coordinateCommentRefMap(coordinateCommentRefMap)
+  {
+  }
+protected:
+  // attribute: only ID ?
+  IWORKXMLContextPtr_t element(int name) final;
+
+  std::map<std::pair<unsigned,unsigned>,ID_t> &m_coordinateCommentRefMap;
+};
+
+IWORKXMLContextPtr_t CellCommentMapping::element(int name)
+{
+  switch (name)
+  {
+  case IWORKToken::pair | IWORKToken::NS_URI_SFA :
+    return std::make_shared<CellCommentMappingPair>(getState(), m_coordinateCommentRefMap);
+  default:
+    return IWORKXMLEmptyContextBase::element(name);
+  }
+}
+
+}
+
 namespace
 {
 // cell format ?
@@ -177,14 +319,11 @@ void CellContextBase::emitCell(const bool covered)
   if (tableData->m_cellMove)
   {
     const unsigned ct = get(tableData->m_cellMove);
-    if (0x80 > ct)
+    tableData->m_column += ct;
+    if (tableData->m_column>=256)
     {
-      tableData->m_column += ct;
-    }
-    else
-    {
-      ++tableData->m_row;
-      tableData->m_column -= (0x100 - ct);
+      tableData->m_row+=(tableData->m_column/256);
+      tableData->m_column%=256;
     }
   }
   else
@@ -514,6 +653,41 @@ void NElement::attribute(const int name, const char *const value)
 namespace
 {
 
+class OElement : public CellContextBase
+{
+public:
+  explicit OElement(IWORKXMLParserState &state, bool isResult=false);
+
+private:
+  void attribute(int name, const char *value) override;
+  void endOfElement() override;
+};
+
+OElement::OElement(IWORKXMLParserState &state, bool isResult)
+  : CellContextBase(state, isResult)
+{
+}
+
+void OElement::attribute(const int name, const char *const value)
+{
+  switch (name)
+  {
+  case IWORKToken::ho | IWORKToken::NS_URI_SF : // horizontal overlap?
+    break;
+  default :
+    CellContextBase::attribute(name, value);
+  }
+}
+
+void OElement::endOfElement()
+{
+  emitCell(true);
+}
+}
+
+namespace
+{
+
 class TElement : public CellContextBase
 {
 public:
@@ -739,99 +913,29 @@ IWORKXMLContextPtr_t GroupingElement::element(int name)
 namespace
 {
 
-class PmCtElement : public IWORKXMLEmptyContextBase
+struct CellData
 {
-public:
-  explicit PmCtElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap, const boost::optional<std::string> &id);
+  CellData()
+    : m_content()
+    , m_dateTime()
+    , m_formula()
+    , m_style()
+    , m_type()
+    , m_text()
+  {
+  }
 
-private:
-  void attribute(int name, const char *value) override;
-
-private:
-  IWORKContentMap_t &m_contentMap;
-  const boost::optional<std::string> &m_id;
+  boost::optional<std::string> m_content;
+  boost::optional<IWORKDateTimeData> m_dateTime;
+  IWORKFormulaPtr_t m_formula;
+  IWORKStylePtr_t m_style;
+  IWORKCellType m_type;
+  IWORKTextPtr_t m_text;
 };
-
-PmCtElement::PmCtElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap, const boost::optional<std::string> &id)
-  : IWORKXMLEmptyContextBase(state)
-  , m_contentMap(contentMap)
-  , m_id(id)
-{
-}
-
-void PmCtElement::attribute(const int name, const char *const value)
-{
-  if (name == (IWORKToken::s | IWORKToken::NS_URI_SFA) && m_id)
-    m_contentMap[get(m_id)] = value;
-}
-
 }
 
 namespace
 {
-
-class PmTElement : public IWORKXMLEmptyContextBase
-{
-public:
-  explicit PmTElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap);
-
-private:
-  IWORKXMLContextPtr_t element(int name) override;
-
-private:
-  IWORKContentMap_t &m_contentMap;
-};
-
-PmTElement::PmTElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap)
-  : IWORKXMLEmptyContextBase(state)
-  , m_contentMap(contentMap)
-{
-}
-
-IWORKXMLContextPtr_t PmTElement::element(const int name)
-{
-  if (name == (IWORKToken::ct | IWORKToken::NS_URI_SF))
-    return std::make_shared<PmCtElement>(getState(), m_contentMap, getId());
-
-  return IWORKXMLContextPtr_t();
-}
-
-}
-
-namespace
-{
-
-class MenuChoicesElement : public IWORKXMLElementContextBase
-{
-public:
-  explicit MenuChoicesElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap);
-
-private:
-  IWORKXMLContextPtr_t element(int name) override;
-
-private:
-  IWORKContentMap_t &m_contentMap;
-};
-
-MenuChoicesElement::MenuChoicesElement(IWORKXMLParserState &state, IWORKContentMap_t &contentMap)
-  : IWORKXMLElementContextBase(state)
-  , m_contentMap(contentMap)
-{
-}
-
-IWORKXMLContextPtr_t MenuChoicesElement::element(int name)
-{
-  if (name == (IWORKToken::t | IWORKToken::NS_URI_SF))
-    return std::make_shared<PmTElement>(getState(), m_contentMap);
-
-  return IWORKXMLContextPtr_t();
-}
-
-}
-
-namespace
-{
-
 class PmElement : public CellContextBase
 {
 public:
@@ -842,7 +946,7 @@ private:
   void endOfElement() override;
 
 private:
-  IWORKContentMap_t m_contentMap;
+  std::map<ID_t,CellData> m_contentMap;
   boost::optional<ID_t> m_ref;
 };
 
@@ -853,6 +957,85 @@ PmElement::PmElement(IWORKXMLParserState &state)
 {
 }
 
+}
+
+namespace
+{
+class TElementInMenu : public TElement
+{
+public:
+  explicit TElementInMenu(IWORKXMLParserState &state, std::map<ID_t,CellData> &contentMap);
+
+private:
+  void endOfElement() override;
+
+private:
+  std::map<ID_t,CellData> &m_contentMap;
+};
+
+TElementInMenu::TElementInMenu(IWORKXMLParserState &state, std::map<ID_t,CellData> &contentMap)
+  : TElement(state)
+  , m_contentMap(contentMap)
+{
+}
+
+void TElementInMenu::endOfElement()
+{
+  const IWORKTableDataPtr_t tableData = getState().m_tableData;
+  if (getId())
+  {
+    // save data
+    auto &data=m_contentMap[*getId()];
+    data.m_content=tableData->m_content;
+    data.m_dateTime=tableData->m_dateTime;
+    data.m_formula=tableData->m_formula;
+    data.m_type=tableData->m_type;
+    data.m_text=getState().m_currentText;
+  }
+
+  // reset content attributes
+  getState().m_currentText.reset();
+  tableData->m_content.reset();
+  tableData->m_dateTime.reset();
+  tableData->m_formula.reset();
+  tableData->m_style.reset();
+  tableData->m_type = IWORK_CELL_TYPE_TEXT;
+}
+}
+
+namespace
+{
+
+class MenuChoicesElement : public IWORKXMLElementContextBase
+{
+public:
+  explicit MenuChoicesElement(IWORKXMLParserState &state, std::map<ID_t,CellData> &contentMap);
+
+private:
+  IWORKXMLContextPtr_t element(int name) override;
+
+private:
+  std::map<ID_t,CellData> &m_contentMap;
+};
+
+MenuChoicesElement::MenuChoicesElement(IWORKXMLParserState &state, std::map<ID_t,CellData> &contentMap)
+  : IWORKXMLElementContextBase(state)
+  , m_contentMap(contentMap)
+{
+}
+
+IWORKXMLContextPtr_t MenuChoicesElement::element(int name)
+{
+  if (name == (IWORKToken::t | IWORKToken::NS_URI_SF))
+    return std::make_shared<TElementInMenu>(getState(), m_contentMap);
+
+  return IWORKXMLContextPtr_t();
+}
+
+}
+
+namespace
+{
 IWORKXMLContextPtr_t PmElement::element(int name)
 {
   switch (name)
@@ -875,15 +1058,25 @@ void PmElement::endOfElement()
 {
   if (m_ref)
   {
-    const IWORKContentMap_t::const_iterator it = m_contentMap.find(get(m_ref));
+    auto const it = m_contentMap.find(get(m_ref));
     if (m_contentMap.end() != it)
     {
-      getState().m_tableData->m_content = it->second;
-      getState().m_tableData->m_type = IWORK_CELL_TYPE_TEXT;
+      // restore data
+      const IWORKTableDataPtr_t tableData = getState().m_tableData;
+      auto const &data=it->second;
+      tableData->m_content=data.m_content;
+      tableData->m_dateTime=data.m_dateTime;
+      tableData->m_formula=data.m_formula;
+      tableData->m_type=data.m_type;
+      getState().m_currentText=data.m_text;
+    }
+    else
+    {
+      ETONYEK_DEBUG_MSG(("PmElement::endOfElement[IWORKTabularModelElement.cpp]: can not found %s references\n", m_ref->c_str()));
     }
   }
+  CellContextBase::endOfElement();
 }
-
 }
 
 namespace
@@ -1472,6 +1665,8 @@ IWORKXMLContextPtr_t DatasourceElement::element(const int name)
     return std::make_shared<GroupingElement>(getState());
   case IWORKToken::n | IWORKToken::NS_URI_SF :
     return std::make_shared<NElement>(getState());
+  case IWORKToken::o | IWORKToken::NS_URI_SF :
+    return std::make_shared<OElement>(getState());
   case IWORKToken::pm | IWORKToken::NS_URI_SF :
     return std::make_shared<PmElement>(getState());
   case IWORKToken::s | IWORKToken::NS_URI_SF :
@@ -1817,6 +2012,7 @@ IWORKTabularModelElement::IWORKTabularModelElement(IWORKXMLParserState &state, b
   , m_headerColumns()
   , m_headerRows()
   , m_footerRows()
+  , m_coordinateCommentRefMap()
 {
 }
 
@@ -1848,12 +2044,6 @@ void IWORKTabularModelElement::attribute(const int name, const char *value)
   }
 }
 
-void IWORKTabularModelElement::endOfAttributes()
-{
-  if (m_tableId && m_tableName)
-    (*(getState().m_tableNameMap))[get(m_tableId)] = get(m_tableName);
-}
-
 void IWORKTabularModelElement::startOfElement()
 {
   getState().m_tableData = std::make_shared<IWORKTableData>();
@@ -1874,6 +2064,7 @@ IWORKXMLContextPtr_t IWORKTabularModelElement::element(const int name)
     return std::make_shared<IWORKRefContext>(getState(), m_styleRef);
 
   case IWORKToken::cell_comment_mapping | IWORKToken::NS_URI_SF :
+    return std::make_shared<CellCommentMapping>(getState(), m_coordinateCommentRefMap);
   case IWORKToken::error_warning_mapping | IWORKToken::NS_URI_SF :
   case IWORKToken::filterset | IWORKToken::NS_URI_SF :
   case IWORKToken::grouping_order | IWORKToken::NS_URI_SF :
@@ -1890,6 +2081,30 @@ IWORKXMLContextPtr_t IWORKTabularModelElement::element(const int name)
 
 void IWORKTabularModelElement::endOfElement()
 {
+  if (m_tableName && !get(m_tableName).empty())
+  {
+    auto &tableMap=*getState().m_tableNameMap;
+    auto finalName=get(m_tableName);
+    if (tableMap.find(finalName)!=tableMap.end())
+    {
+      ETONYEK_DEBUG_MSG(("IWORKTabularModelElement::endElement: a table with name %s already exists\n", finalName.c_str()));
+      // let create an unique name
+      int id=0;
+      while (true)
+      {
+        std::stringstream s;
+        s << finalName << "_" << ++id;
+        if (tableMap.find(s.str())!=tableMap.end()) continue;
+        finalName=s.str();
+        break;
+      }
+    }
+    tableMap[finalName]=finalName;
+    if (m_tableId)
+      tableMap[get(m_tableId)] = finalName;
+    if (bool(getState().m_currentTable))
+      getState().m_currentTable->setName(finalName);
+  }
   if (bool(getState().m_currentTable))
   {
     IWORKStylePtr_t style;
@@ -1899,6 +2114,23 @@ void IWORKTabularModelElement::endOfElement()
     getState().m_currentTable->setHeaders(
       get_optional_value_or(m_headerColumns, 0), get_optional_value_or(m_headerRows, 0),
       get_optional_value_or(m_footerRows, 0));
+    if (!m_coordinateCommentRefMap.empty())
+    {
+      auto &dico=getState().getDictionary().m_tableComments;
+      for (auto const &it : m_coordinateCommentRefMap)
+      {
+        auto dIt=dico.find(it.second);
+        if (dIt==dico.end() || !dIt->second)
+        {
+          ETONYEK_DEBUG_MSG(("IWORKTabularModelElement::endElement: can not find comment with name %s\n", it.second.c_str()));
+          continue;
+        }
+        IWORKOutputElements noteElements;
+        // FIXME: we must set the correct sheetref before calling this
+        dIt->second->draw(noteElements);
+        getState().m_currentTable->setComment(it.first.first, it.first.second, noteElements);
+      }
+    }
   }
   if (!m_isDefinition)
     return;
